package comp.nus.sociallinks.spark.main

import spark._
import spark.SparkContext._


class ComputeTriangle(masterURL:String, val inFilePath: String) {

  //get all the triads based on the adjancencyList
  //a,(b,c,d) --> ((b,c),a),((b,d),a),((c,d),a) 
  def getTriadsFromAdjacencyList(adjacencyList: (String, Seq[String])): List[((String, String), String)] = {
    val vertex = adjacencyList._1
    val neighbors = adjacencyList._2.toSet.toList
    val result = Util.doubleIterate(vertex, neighbors)(getSingleTriad)
    result
  }

  //join the vertex with two other vertex
  //a,b,c --> ((b,c),a)
  def getSingleTriad(vertex: String, in: String, out: String): ((String, String), String) = {
    val pair = if (in.toInt < out.toInt) {
      (in, out)
    } else {
      (out, in)
    }
    (pair, vertex)
  }

  /*
   * add number of triangles to vertex pairs
   */
  def computeTriangles(vertexPairs: RDD[(String, String)]): RDD[((String, String), Int)] = {
    val triads = vertexPairs.groupByKey()
      .flatMap(adjacencyList => getTriadsFromAdjacencyList(adjacencyList))

    val edgeMaps = vertexPairs.map((_, ""))

    val triangles = triads.join(edgeMaps)

    val edgetriangles = triangles.flatMap(triangle => {
      //a<b<c
      val edge = triangle._1
      val b = edge._1
      val c = edge._2
      val a = triangle._2._1
      val list = ((a, b), 1) :: ((a, c), 1) :: ((b, c), 1) :: List()
      list
    })
      .reduceByKey(_ + _)

    edgetriangles
  }

  def run(): RDD[((String,String),Int)] = {
    val spark = new SparkContext(masterURL, "ComputeTriangle")
    
    //data is stored as the edge list
    //1\t2
    //3\t1
    //....
    val edges = spark.textFile(inFilePath).cache

    //from edge list to vertex pairs, vertex with smaller id in the front
    //String --> (String, String)
    val vertexPairs = edges.map(line => {
      val fields = line.split("\t")
      val vertex1 = fields(0).trim
      val vertex2 = fields(1).trim
      val pair = if (vertex1.toInt < vertex2.toInt) {
        (vertex1, vertex2)
      } else {
        (vertex2, vertex1)
      }
      (pair)
    })
    val edgeTriangles = computeTriangles(vertexPairs)
    edgeTriangles
  }

}

object ComputeTriangle {
  def main(args: Array[String]) {

    if (args.length < 2) {
      System.err.println("Usage: ComputeTriangle <master> <infile> [<outfile>]")
      System.exit(1)
    }

    val runner = new ComputeTriangle(args(0), args(1))

    val triangles = runner.run
    
    if (args.length == 3) {
      triangles.saveAsTextFile(args(2))
    } else {
      triangles.saveAsTextFile("data/out")
    }
  }
} 


